// Jim's simple maze, adapted from my Javascript maze...
//
// Jim Bumgardner 2004
//

W = 320
H = 240

var cells;
var cw=0,ch=0;

var nx = new Array(1,0,-1,0);
var ny = new Array(0,-1,0,1);
var nse = new Array(0x11,0x12,0x10,0x10);
var nde = new Array(0x10,0x10,0x11,0x12);
var frameColor = 0x000000;
var wallColor = 0x666666;
var me = this;

makemaze(16,12);

++initCtr;
if (initCtr > 1)
	trace("Script called twice");

function drawFrame()
{
	// trace("draw frame");
	var tw = me.tw;
	var th = me.th;
	var cw = me.cw;
	var ch = me.ch;

	this.clear();

	this.lineStyle(4,me.frameColor,100);
	this.moveTo(tw,0);
	this.lineTo(tw * cw, 			0);
	this.lineTo(tw * cw, 			th * ch);
	this.moveTo(tw * cw - tw, th * ch);
	this.lineTo(0, 						th * ch);
	this.lineTo(0,						0);
}

function drawCell()
{
	this.clear();
	this.lineStyle(4,me.wallColor);
	var cell = this.cell;
	if (this.cell == 0) {
	}
	if ( this.cell & 1) {
		this.moveTo(me.tw,0);
		this.lineTo(me.tw,me.th);
	}
	if ( this.cell & 2) {
		this.moveTo(me.tw,0);
		this.lineTo(0,-.25);
	}
}

function makemaze(width, height)
{
	cw = width;
	ch = height;

	var margin = .10;
	TW = (W*(1-margin*2))/cw;
	TH = (H*(1-margin*2))/ch; // height tends to be shorter...

	var OX = W*margin;
	var OY = H*margin;
	
	// trace("TH = " + TH);
	// trace("OX = " + OX);

	cells = new Array(ch);
	for (var y = 0; y < ch; ++y)
	{
		cells[y] = new Array(cw);
		for (var x = 0; x < cw; ++x)
			cells[y][x] = 0x13;
	}
	// create entrance and exit
	cells[0][0] = 0x11;
	cells[ch-1][cw-1] = 0x13;

	recursemake(Math.floor(cw/2), Math.floor(ch/2));
	// trace("x Recursion Complete");

	for (var y = 0; y < ch; ++y)
	{
		for (var x = 0; x < cw; ++x) 
		{
			var nam = "maze_" + y + "_" + x;
			var mc = this.createEmptyMovieClip(nam,n);
			mc.onEnterFrame = drawCell;
			mc.cell = cells[y][x];
			mc._x = OX+TW*x;
			mc._y = OY+TH*y;
			mc.gotoAndStop(1);
			n++;
		}
	}
	var mc = this.createEmptyMovieClip("frame",n++);
	mc.onEnterFrame = drawFrame;
	mc.gotoAndStop(1);
	mc._x = OX;
	mc._y = OY;
	mc = this.attachMovie("enter","enter",n++);
	var as = mc._height / mc._width;
	var sc = TW * mc._width/100;	// arrow is 100
	mc._width = sc;
	mc._height = sc*as;
	mc._x = OX+.25;
	mc._y = OY;

	mc = this.attachMovie("leave","leave",n++);
	as = mc._height / mc._width;
	sc = TW* mc._width/100;
	mc._width = sc;
	mc._height = sc*as;
	mc._x = OX+(cw-1)*TW;
	mc._y = OY+ch*TH;


}

function irand(n)
{
	return Math.floor(Math.random()*n);
}

function recursemake(x, y)
{
	var cstack = new Array(0);
	while (1)
	{
		var nn = 0;
		var n = -1;
		
		if (x+1 < cw && (cells[y][x+1] & 0x10) > 0 && irand(++nn) < 1)
			n = 0;
		if (y-1 >= 0 && (cells[y-1][x] & 0x10) > 0 && irand(++nn) < 1)
			n = 1;
		if (x-1 >= 0 && (cells[y][x-1] & 0x10) > 0 && irand(++nn) < 1)
			n = 2;
		if (y+1 < ch && (cells[y+1][x] & 0x10) > 0 && irand(++nn) < 1)
			n = 3;

		if (nn > 0)
		{
			// clear path to neighbor
			cells[y][x] &= ~nse[n];  
			cells[y+ny[n]][x+nx[n]] &= ~nde[n];
			// recurse on neighbor
			cstack[cstack.length] = (x << 8) | y;
			x += nx[n];
			y += ny[n];
		}
		else if (cstack.length > 0) 
		{
			// randomly swap in lower element, rather than popping off top
			// this is "hunt and kill" algorithm
			//
			// if we just used top element, it would be "recursive backtracker"
			// which makes snakier mazes
			//
			var r = irand(cstack.length);
			x = cstack[r] >> 8;
			y = cstack[r] & 0xFF;
			cstack[r] = cstack[cstack.length-1];
			cstack.length--;			
		}
		else
			break;
	}
}


